@@ -1,6 +1,6 @@
 #undef NAUTILUS_BUILD_MESSAGE
+#undef NAUTILUS_SOURCE_DIRECTORY
 #undef EAZEL_BUILD_TIMESTAMP
-#undef EAZEL_SERVICES
 #undef ENABLE_NLS
 #undef ENABLE_PROFILER
 #undef ENABLE_INSTALLER
@@ -8,12 +8,10 @@
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_MEDUSA
-#undef HAVE_PAM
-#undef HAVE_RPM
-#undef HAVE_RPM_30
-#undef HAVE_RPM_40
+#undef HAVE_LIBJPEG
 #undef HAVE_STPCPY
 #undef HAVE_LIBBZ2
+#undef HAVE_AMMONITE
 #undef bzCompress
 #undef bzCompressInit
 #undef bzDecompress
@@ -25,9 +23,3 @@
 #undef bzopen
 #undef bzread
 #undef bzwrite
-#undef HAVE_DEV
-#undef HAVE_VOL_DEV
-#undef HAVE_VOL
-#undef HAVE_ETC_MNTTAB
-#undef HAVE_PROC_MOUNTS
-#undef HAVE_PROC_PROCESS_FILES
