@@ -28,25 +28,22 @@
 #include <config.h>
 #include "nautilus-throbber.h"
 
-#include <libgnome/gnome-defs.h>
-#include <bonobo.h>
-
-#include <math.h>
-#include <gnome.h>
-#include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtksignal.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtksignal.h>
+#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-pixmap.h>
-#include <libnautilus-extensions/nautilus-gtk-macros.h>
-#include <libnautilus-extensions/nautilus-gtk-extensions.h>
-#include <libnautilus-extensions/nautilus-glib-extensions.h>
-#include <libnautilus-extensions/nautilus-global-preferences.h>
-#include <libnautilus-extensions/nautilus-icon-factory.h>
-#include <libnautilus-extensions/nautilus-file-utilities.h>
-#include <libnautilus-extensions/nautilus-theme.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
+#include <eel/eel-glib-extensions.h>
+#include <libnautilus-private/nautilus-global-preferences.h>
+#include <eel/eel-gtk-extensions.h>
+#include <eel/eel-gtk-macros.h>
+#include <libnautilus-private/nautilus-icon-factory.h>
+#include <libnautilus-private/nautilus-theme.h>
+#include <libnautilus/nautilus-bonobo-workarounds.h>
+#include <math.h>
 
 #define THROBBER_DEFAULT_TIMEOUT 100	/* Milliseconds Per Frame */
 
@@ -85,7 +82,7 @@ static void	nautilus_throbber_size_allocate		 (GtkWidget *widget, GtkAllocation
 static void	nautilus_throbber_size_request		 (GtkWidget *widget, GtkRequisition *requisition);
 static void     nautilus_throbber_remove_update_callback (NautilusThrobber *throbber);
 
-NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusThrobber, nautilus_throbber, GTK_TYPE_EVENT_BOX)
+EEL_DEFINE_CLASS_BOILERPLATE (NautilusThrobber, nautilus_throbber, GTK_TYPE_EVENT_BOX)
 
 static void
 nautilus_throbber_initialize_class (NautilusThrobberClass *throbber_class)
@@ -188,21 +185,23 @@ static void
 nautilus_throbber_destroy (GtkObject *object)
 {
 	NautilusThrobber *throbber = NAUTILUS_THROBBER (object);
+
+	nautilus_bonobo_object_force_destroy_at_idle (throbber->details->control);
 	
 	nautilus_throbber_remove_update_callback (throbber);
 	nautilus_throbber_unload_images (throbber);
 
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_THEME,
-					      nautilus_throbber_theme_changed,
-					      object);
-
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_THEME,
+					 nautilus_throbber_theme_changed,
+					 object);
+	
 	if (throbber->details->property_bag) {
 		bonobo_object_unref (BONOBO_OBJECT (throbber->details->property_bag));
 	}
 	
 	g_free (throbber->details);
 
-	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 BonoboObject*
@@ -247,6 +246,13 @@ get_throbber_dimensions (NautilusThrobber *throbber, int *throbber_width, int* t
 	*throbber_height = current_height;
 }
 
+static void
+null_pointer_callback (GtkObject *object,
+		       gpointer callback_data)
+{
+	* (gpointer *) callback_data = NULL;
+}
+
 /* initialize the throbber */
 static void
 nautilus_throbber_initialize (NautilusThrobber *throbber)
@@ -277,7 +283,12 @@ nautilus_throbber_initialize (NautilusThrobber *throbber)
 	}
 	
 	/* make the bonobo control */
-	throbber->details->control = (BonoboObject*) bonobo_control_new (widget);
+	throbber->details->control = BONOBO_OBJECT (bonobo_control_new (widget));
+	gtk_signal_connect_while_alive (GTK_OBJECT (throbber->details->control),
+					"destroy",
+					null_pointer_callback,
+					&throbber->details->control,
+					GTK_OBJECT (throbber));
 	
 	/* attach a property bag with the configure property */
 	throbber->details->property_bag = bonobo_property_bag_new (get_bonobo_properties, 
@@ -295,9 +306,9 @@ nautilus_throbber_initialize (NautilusThrobber *throbber)
 	gtk_widget_show (widget);
 	
 	/* add a callback for when the theme changes */
-	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_THEME,
-					  nautilus_throbber_theme_changed,
-					  throbber);
+	eel_preferences_add_callback (NAUTILUS_PREFERENCES_THEME,
+				      nautilus_throbber_theme_changed,
+				      throbber);
 }
 
 /* allocate a new throbber */
@@ -419,7 +430,7 @@ nautilus_throbber_map (GtkWidget *widget)
 	
 	throbber = NAUTILUS_THROBBER (widget);
 	
-	NAUTILUS_CALL_PARENT (GTK_WIDGET_CLASS, map, (widget));
+	EEL_CALL_PARENT (GTK_WIDGET_CLASS, map, (widget));
 	throbber->details->ready = TRUE;
 }
 
@@ -627,7 +638,7 @@ nautilus_throbber_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	int throbber_width, throbber_height;
 	NautilusThrobber *throbber = NAUTILUS_THROBBER (widget);
 
-	NAUTILUS_CALL_PARENT (GTK_WIDGET_CLASS, size_allocate, (widget, allocation));
+	EEL_CALL_PARENT (GTK_WIDGET_CLASS, size_allocate, (widget, allocation));
 
 	get_throbber_dimensions (throbber, &throbber_width, &throbber_height);
 	
