@@ -28,19 +28,16 @@
 #include <config.h>
 #include "nautilus-tree-view-dnd.h"
 
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmain.h>
 #include <gtk/gtkdnd.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtksignal.h>
 #include <libgnome/gnome-i18n.h>
-#include <libnautilus-extensions/nautilus-glib-extensions.h>
-#include <libnautilus-extensions/nautilus-gtk-extensions.h>
-#include <libnautilus-extensions/nautilus-drag.h>
-#include <libnautilus-extensions/nautilus-file.h>
-#include <libnautilus-extensions/nautilus-background.h>
-#include <libnautilus-extensions/nautilus-file-operations.h>
-
-#define nopeMATHIEU_DEBUG 1
-
+#include <eel/eel-background.h>
+#include <libnautilus-private/nautilus-file-operations.h>
+#include <libnautilus-private/nautilus-file.h>
+#include <eel/eel-glib-extensions.h>
+#include <eel/eel-gtk-extensions.h>
 
 /* This constant is zero because right now it does not seem we need
    extra delay on horizontal-only auto-scroll. However, it's left in
@@ -104,11 +101,11 @@ static char    *nautilus_tree_view_find_drop_target   (NautilusTreeView
 						       int                       y);
 static char    *nautilus_tree_view_get_drag_uri           (NautilusTreeView      *tree_view);
 
-static void     nautilus_tree_view_expand_or_collapse_row (NautilusCTree         *tree, 
+static void     nautilus_tree_view_expand_or_collapse_row (EelCTree         *tree, 
 							   int                    row);
 static gboolean nautilus_tree_view_is_tree_node_directory (NautilusTreeView      *tree_view,
-							   NautilusCTreeNode     *node);
-static NautilusCTreeNode *nautilus_tree_view_tree_node_at (NautilusTreeView      *tree_view,
+							   EelCTreeNode     *node);
+static EelCTreeNode *nautilus_tree_view_tree_node_at (NautilusTreeView      *tree_view,
 							   int                    x, 
 							   int                    y);
 
@@ -126,9 +123,6 @@ static void    nautilus_tree_view_receive_dropped_icons   (NautilusTreeView
 							   int                    x, 
 							   int                    y);
 
-static void  nautilus_tree_view_make_prelight_if_file_operation (NautilusTreeView *tree_view, 
-								 int x, 
-								 int y);
 static void  nautilus_tree_view_ensure_drag_data (NautilusTreeView *tree_view,
 						  GdkDragContext *context,
 						  guint32 time);
@@ -140,7 +134,7 @@ static void  nautilus_tree_view_get_drop_action (NautilusTreeView    *tree_view,
 						 int                 *default_action,
 						 int                 *non_default_action);
 static void  nautilus_tree_view_collapse_all    (NautilusTreeView    *tree_view,
-						 NautilusCTreeNode   *current_node);
+						 EelCTreeNode   *current_node);
 
 static void  nautilus_tree_view_set_dnd_icon    (NautilusTreeView    *tree_view,
 						 GdkDragContext      *context);
@@ -149,10 +143,12 @@ static void nautilus_tree_view_drag_destroy     (NautilusTreeView    *tree_view)
 static void nautilus_tree_view_drag_destroy_real (NautilusTreeView   *tree_view);
 
 static GtkTargetEntry nautilus_tree_view_dnd_target_table[] = {
-	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
-	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST }
+	{ EEL_ICON_DND_GNOME_ICON_LIST_TYPE, 0, EEL_ICON_DND_GNOME_ICON_LIST },
+	{ EEL_ICON_DND_URI_LIST_TYPE, 0, EEL_ICON_DND_URI_LIST }
 };
 
+static GtkTargetList *nautilus_tree_view_dnd_target_list = NULL;
+
 
 static void 
 tree_view_realize_callback (GtkWidget *widget, gpointer user_data) 
@@ -171,7 +167,7 @@ tree_view_realize_callback (GtkWidget *widget, gpointer user_data)
         gtk_style_ref (new_style);
 
 	/* calculate a new prelighting color */
-	nautilus_gtk_style_shade (&style->bg[GTK_STATE_SELECTED], 
+	eel_gtk_style_shade (&style->bg[GTK_STATE_SELECTED], 
 				  &new_prelight_color, 
 				  1.35);
 	/* set the new color to our special prelighting Style. */
@@ -197,17 +193,17 @@ nautilus_tree_view_init_dnd (NautilusTreeView *view)
 	view->details->dnd = g_new0 (NautilusTreeViewDndDetails, 1);
 	view->details->dnd->expanded_nodes = NULL;
 
-	view->details->dnd->drag_info = g_new0 (NautilusDragInfo, 1);
-	nautilus_drag_init (view->details->dnd->drag_info,
+	view->details->dnd->drag_info = g_new0 (EelDragInfo, 1);
+	eel_drag_init (view->details->dnd->drag_info,
 			    nautilus_tree_view_dnd_target_table,
-			    NAUTILUS_N_ELEMENTS (nautilus_tree_view_dnd_target_table),
+			    EEL_N_ELEMENTS (nautilus_tree_view_dnd_target_table),
 			    NULL);
 
 
 	gtk_drag_dest_set (GTK_WIDGET (view->details->tree), 
 			   0,
 			   nautilus_tree_view_dnd_target_table,
-			   NAUTILUS_N_ELEMENTS (nautilus_tree_view_dnd_target_table),
+			   EEL_N_ELEMENTS (nautilus_tree_view_dnd_target_table),
 			   GDK_ACTION_COPY 
 			   | GDK_ACTION_MOVE 
 			   | GDK_ACTION_LINK 
@@ -274,7 +270,7 @@ nautilus_tree_view_free_dnd (NautilusTreeView *view)
 	if (view->details->dnd->highlight_style != NULL) {
 		gtk_style_unref (view->details->dnd->highlight_style);
 	}
-	nautilus_drag_finalize (view->details->dnd->drag_info);
+	eel_drag_finalize (view->details->dnd->drag_info);
 	g_free (view->details->dnd);
 }
 
@@ -297,7 +293,7 @@ nautilus_tree_view_drag_begin (GtkWidget *widget, GdkDragContext *context,
 
 	dnd->drag_info->got_drop_data_type = FALSE;
 
-	nautilus_tree_view_set_dnd_icon (NAUTILUS_TREE_VIEW (tree_view), context);
+	nautilus_tree_view_set_dnd_icon (tree_view, context);
 }
 
 
@@ -336,7 +332,7 @@ nautilus_tree_view_drag_motion (GtkWidget *widget, GdkDragContext *context,
 {
 	NautilusTreeView *tree_view;
 	NautilusTreeViewDndDetails *dnd;
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 
 	int resulting_action, default_action, non_default_action;
 
@@ -357,19 +353,16 @@ nautilus_tree_view_drag_motion (GtkWidget *widget, GdkDragContext *context,
 	/* prelight depending on the type of drop. */
 	if (drag_info->got_drop_data_type) {
 		switch (drag_info->data_type) {
-		case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
-		case NAUTILUS_ICON_DND_URI_LIST:
+		case EEL_ICON_DND_GNOME_ICON_LIST:
+		case EEL_ICON_DND_URI_LIST:
 
-			nautilus_ctree_set_prelight (NAUTILUS_CTREE (tree_view->details->tree), 
+			eel_ctree_set_prelight (EEL_CTREE (tree_view->details->tree), 
 						     y);
 			
-			nautilus_tree_view_make_prelight_if_file_operation (NAUTILUS_TREE_VIEW (tree_view), 
-									    x, y);
-
 			break;
-		case NAUTILUS_ICON_DND_KEYWORD:	
-		case NAUTILUS_ICON_DND_COLOR:
-		case NAUTILUS_ICON_DND_BGIMAGE:	
+		case EEL_ICON_DND_KEYWORD:	
+		case EEL_ICON_DND_COLOR:
+		case EEL_ICON_DND_BGIMAGE:	
 		default:
 			break;
 		}
@@ -382,7 +375,7 @@ nautilus_tree_view_drag_motion (GtkWidget *widget, GdkDragContext *context,
 	nautilus_tree_view_get_drop_action  (tree_view, context, x, y, 
 					     &default_action, 
 					     &non_default_action);
-	resulting_action = nautilus_drag_modifier_based_action (default_action,
+	resulting_action = eel_drag_modifier_based_action (default_action,
 								non_default_action);
 	gdk_drag_status (context, resulting_action, time);
 
@@ -394,6 +387,30 @@ nautilus_tree_view_drag_motion (GtkWidget *widget, GdkDragContext *context,
 	return TRUE;
 }
 
+/* FIXME bugzilla.eazel.com 7445: Needs to become a shared function */
+static void
+get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context, guint32 time)
+{
+	GList *target;
+
+	if (nautilus_tree_view_dnd_target_list == NULL)
+		nautilus_tree_view_dnd_target_list = gtk_target_list_new (nautilus_tree_view_dnd_target_table,
+									  EEL_N_ELEMENTS (nautilus_tree_view_dnd_target_table));
+
+	for (target = context->targets; target != NULL; target = target->next) {
+		guint dummy_info;
+		GdkAtom target_atom = GPOINTER_TO_UINT (target->data);
+
+		if (gtk_target_list_find (nautilus_tree_view_dnd_target_list, 
+					  target_atom,
+					  &dummy_info)) {
+			gtk_drag_get_data (widget, context,
+					   target_atom,
+					   time);
+			break;
+		}
+	}
+}
 
 static gboolean 
 nautilus_tree_view_drag_drop (GtkWidget *widget,
@@ -413,9 +430,7 @@ nautilus_tree_view_drag_drop (GtkWidget *widget,
 	 * drop occured.
 	 */
 	dnd->drag_info->drop_occured = TRUE;
-	gtk_drag_get_data (GTK_WIDGET (widget), context,
-			   GPOINTER_TO_INT (context->targets->data),
-			   time);
+	get_data_on_first_target_we_support (widget, context, time);
 
 	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget),
 				      "drag_drop");
@@ -431,7 +446,7 @@ nautilus_tree_view_drag_data_received (GtkWidget *widget,
 				       guint info, guint time)
 {
 	NautilusTreeViewDndDetails *dnd;
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 	NautilusTreeView *tree_view;
 
 	tree_view = NAUTILUS_TREE_VIEW (gtk_object_get_data (GTK_OBJECT (widget), "tree_view"));
@@ -445,15 +460,15 @@ nautilus_tree_view_drag_data_received (GtkWidget *widget,
 
 		/* save operation for drag motion events */
 		switch (info) {
-		case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
-			drag_info->selection_list = nautilus_drag_build_selection_list (data);
+		case EEL_ICON_DND_GNOME_ICON_LIST:
+			drag_info->selection_list = eel_drag_build_selection_list (data);
 			break;
-		case NAUTILUS_ICON_DND_URI_LIST:
-			drag_info->selection_list = nautilus_drag_build_selection_list (data);
+		case EEL_ICON_DND_URI_LIST:
+			drag_info->selection_list = eel_drag_build_selection_list (data);
 			break;
-		case NAUTILUS_ICON_DND_COLOR:
-		case NAUTILUS_ICON_DND_BGIMAGE:	
-		case NAUTILUS_ICON_DND_KEYWORD:	
+		case EEL_ICON_DND_COLOR:
+		case EEL_ICON_DND_BGIMAGE:	
+		case EEL_ICON_DND_KEYWORD:	
 		default:
 			/* we do not want to support any of the 3 above */
 			break;
@@ -463,16 +478,16 @@ nautilus_tree_view_drag_data_received (GtkWidget *widget,
 	if (drag_info->drop_occured) {
 		/* drop occured: do actual operations on the data */
 		switch (info) {
-		case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
-		case NAUTILUS_ICON_DND_URI_LIST:
+		case EEL_ICON_DND_GNOME_ICON_LIST:
+		case EEL_ICON_DND_URI_LIST:
 			nautilus_tree_view_receive_dropped_icons
 				(NAUTILUS_TREE_VIEW (tree_view),
 				 context, x, y);
 			gtk_drag_finish (context, TRUE, FALSE, time);
 			break;
-		case NAUTILUS_ICON_DND_COLOR:
-		case NAUTILUS_ICON_DND_BGIMAGE:
-		case NAUTILUS_ICON_DND_KEYWORD:
+		case EEL_ICON_DND_COLOR:
+		case EEL_ICON_DND_BGIMAGE:
+		case EEL_ICON_DND_KEYWORD:
 		default:
 			gtk_drag_finish (context, FALSE, FALSE, time);
 		}
@@ -538,12 +553,12 @@ static int
 nautilus_tree_view_button_press (GtkWidget *widget, GdkEventButton *event)
 {
 	int retval;
-	GtkCList *clist;
+	EelCList *clist;
 	NautilusTreeView *tree_view;
 		
 	int press_row, press_column, on_row;
 
-	clist = GTK_CLIST (widget);
+	clist = EEL_CLIST (widget);
 	retval = FALSE;
 
 	if (event->window != clist->clist_window)
@@ -551,18 +566,18 @@ nautilus_tree_view_button_press (GtkWidget *widget, GdkEventButton *event)
 
 	tree_view = NAUTILUS_TREE_VIEW (gtk_object_get_data (GTK_OBJECT (widget), "tree_view"));
 
-	on_row = gtk_clist_get_selection_info (GTK_CLIST (tree_view->details->tree), 
-					       event->x, 
-					       event->y, 
-					       &press_row, &press_column);
+	on_row = eel_clist_get_selection_info (clist, 
+						    event->x, 
+						    event->y, 
+						    &press_row, &press_column);
 	if (on_row == 0) {
 		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button-press-event");
 		return FALSE;
 	}
 
-	if (nautilus_ctree_is_hot_spot (NAUTILUS_CTREE (widget), event->x, event->y)) {		
-		NautilusCTreeRow *ctree_row;
-		NautilusCTreeNode *node;
+	if (eel_ctree_is_hot_spot (EEL_CTREE (widget), event->x, event->y)) {		
+		EelCTreeRow *ctree_row;
+		EelCTreeNode *node;
 		
 		tree_view->details->dnd->press_x = event->x;
 		tree_view->details->dnd->press_y = event->y;
@@ -577,9 +592,9 @@ nautilus_tree_view_button_press (GtkWidget *widget, GdkEventButton *event)
 			ctree_row->mouse_down = TRUE;
 			ctree_row->in_hotspot = TRUE;
 
-			node = nautilus_ctree_find_node_ptr (NAUTILUS_CTREE (widget), ctree_row);
+			node = eel_ctree_find_node_ptr (EEL_CTREE (widget), ctree_row);
 			if (node != NULL) {
-				nautilus_ctree_draw_node (NAUTILUS_CTREE (widget), node);
+				eel_ctree_draw_node (EEL_CTREE (widget), node);
 			}
 		}
 	} else {
@@ -607,16 +622,16 @@ static int
 nautilus_tree_view_button_release (GtkWidget *widget, GdkEventButton *event)
 {
 	int retval;
-	GtkCList *clist;
+	EelCList *clist;
 	NautilusTreeView *tree_view;
 	int release_row, release_column, on_row;
 	int distance_squared;
 	gboolean is_still_hot_spot;
 	int press_row, press_column;
-	NautilusCTreeRow *ctree_row;
-	NautilusCTreeNode *node;
+	EelCTreeRow *ctree_row;
+	EelCTreeNode *node;
 
-	clist = GTK_CLIST (widget);
+	clist = EEL_CLIST (widget);
 	retval = FALSE;
 
 	if (event->window != clist->clist_window)
@@ -627,19 +642,19 @@ nautilus_tree_view_button_release (GtkWidget *widget, GdkEventButton *event)
 	
 	/* Set state of spinner.  Use saved dnd x and y as the mouse may have moved out
 	 * of the original row */	
-	on_row = gtk_clist_get_selection_info (GTK_CLIST (tree_view->details->tree), 
-					       tree_view->details->dnd->press_x, 
-					       tree_view->details->dnd->press_y, 
-					       &press_row, &press_column);	
+	on_row = eel_clist_get_selection_info (clist,
+						    tree_view->details->dnd->press_x, 
+						    tree_view->details->dnd->press_y, 
+						    &press_row, &press_column);	
 	ctree_row = ROW_ELEMENT (clist, press_row)->data;
 	if (ctree_row != NULL) {
 		ctree_row->mouse_down = FALSE;
 		ctree_row->in_hotspot = FALSE;
 
 		/* Redraw spinner */
-		node = nautilus_ctree_find_node_ptr (NAUTILUS_CTREE (widget), ctree_row);
+		node = eel_ctree_find_node_ptr (EEL_CTREE (widget), ctree_row);
 		if (node != NULL) {
-			nautilus_ctree_draw_node (NAUTILUS_CTREE (widget), node);
+			eel_ctree_draw_node (EEL_CTREE (widget), node);
 		}
 	}
 
@@ -647,13 +662,13 @@ nautilus_tree_view_button_release (GtkWidget *widget, GdkEventButton *event)
 		* (event->x - tree_view->details->dnd->press_x) +
 		(event->y - tree_view->details->dnd->press_y)
 		* (event->y - tree_view->details->dnd->press_y);
-	is_still_hot_spot = nautilus_ctree_is_hot_spot (NAUTILUS_CTREE(tree_view->details->tree), 
+	is_still_hot_spot = eel_ctree_is_hot_spot (EEL_CTREE(tree_view->details->tree), 
 						   event->x, event->y);
 	
-	on_row = gtk_clist_get_selection_info (GTK_CLIST (tree_view->details->tree), 
-					       event->x, 
-					       event->y, 
-					       &release_row, &release_column);
+	on_row = eel_clist_get_selection_info (clist,
+						    event->x, 
+						    event->y, 
+						    &release_row, &release_column);
 
 	if (on_row == 1) {
 
@@ -663,7 +678,7 @@ nautilus_tree_view_button_release (GtkWidget *widget, GdkEventButton *event)
 			
 			tree_view->details->dnd->pressed_hot_spot = FALSE;
 			
-			nautilus_tree_view_expand_or_collapse_row (NAUTILUS_CTREE(tree_view->details->tree), 
+			nautilus_tree_view_expand_or_collapse_row (EEL_CTREE(tree_view->details->tree), 
 								   release_row);
 		} else if (distance_squared <= RADIUS) {
 			/* we are close from the place we clicked */
@@ -671,8 +686,8 @@ nautilus_tree_view_button_release (GtkWidget *widget, GdkEventButton *event)
 
 			/* Only button 1 triggers a selection */
 			if (event->button == 1) {
-				gtk_clist_select_row (GTK_CLIST (tree_view->details->tree), 
-						      release_row, release_column); 
+				eel_clist_select_row (clist,
+							   release_row, release_column); 
 			}
 		}
 	}
@@ -688,10 +703,10 @@ nautilus_tree_view_button_release (GtkWidget *widget, GdkEventButton *event)
 static int
 nautilus_tree_view_motion_notify (GtkWidget *widget, GdkEventButton *event)
 {
-	GtkCList *clist;
+	EelCList *clist;
 	NautilusTreeView *tree_view;
 
-	clist = GTK_CLIST (widget);
+	clist = EEL_CLIST (widget);
 
 
 	if (event->window != clist->clist_window)
@@ -722,7 +737,7 @@ nautilus_tree_view_motion_notify (GtkWidget *widget, GdkEventButton *event)
 					(GdkEvent *) event);
 
 
-			nautilus_ctree_set_prelight (NAUTILUS_CTREE (tree_view->details->tree), 
+			eel_ctree_set_prelight (EEL_CTREE (tree_view->details->tree), 
 						     tree_view->details->dnd->press_y);
 
 		}
@@ -760,13 +775,11 @@ nautilus_tree_view_motion_notify (GtkWidget *widget, GdkEventButton *event)
 static void 
 nautilus_tree_view_set_dnd_icon (NautilusTreeView *tree_view, GdkDragContext *context)
 {
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-	NautilusCTreeNode *node;
+	GdkPixbuf *pixbuf;
+	EelCTreeNode *node;
 	gchar       *text;
 	guint8       spacing;
-	GdkPixmap   *pixmap_opened;
-	GdkBitmap   *mask_opened;
+	GdkPixbuf   *pixbuf_opened;
 	gboolean     is_leaf;
 	gboolean     expanded;
 	NautilusTreeViewDndDetails *dnd;
@@ -780,83 +793,37 @@ nautilus_tree_view_set_dnd_icon (NautilusTreeView *tree_view, GdkDragContext *co
 						dnd->press_x,
 						dnd->press_y);
 		
-	nautilus_ctree_get_node_info (NAUTILUS_CTREE (tree_view->details->tree),
+	eel_ctree_get_node_info (EEL_CTREE (tree_view->details->tree),
 				      node, &text,
-				      &spacing, &pixmap,
-				      &mask, &pixmap_opened,
-				      &mask_opened, &is_leaf, 
+				      &spacing, &pixbuf,
+				      &pixbuf_opened, &is_leaf, 
 				      &expanded);
 
-	gtk_drag_set_icon_pixmap (context, 
-				  gdk_rgb_get_cmap (),
-				  pixmap, mask,
-				  10, 10);
-}
-
-
-
-static void 
-nautilus_tree_view_make_prelight_if_file_operation (NautilusTreeView *tree_view, 
-						    int x, int y)
-{
-#if 0
-	NautilusTreeViewDndDetails *dnd;
-	NautilusCTreeNode *node;
-	gboolean is_directory;
-
-	g_assert (NAUTILUS_IS_TREE_VIEW (tree_view));
-	dnd = tree_view->details->dnd;
-
-	/* make the thing under us prelighted. */
-	node = nautilus_tree_view_tree_node_at (NAUTILUS_TREE_VIEW (tree_view), x, y);
-	if (node == NULL) {
-		return;
-	}
-	is_directory = nautilus_tree_view_is_tree_node_directory (NAUTILUS_TREE_VIEW (tree_view), node);
-
-	if (!is_directory) {
-		NautilusTreeNode *parent_node, *current_node;
-		current_node = nautilus_tree_view_node_to_model_node (tree_view, node);
-		parent_node = nautilus_tree_node_get_parent (current_node);
-		node = nautilus_tree_view_model_node_to_view_node (tree_view, parent_node);
-	} 
-
-	if (node != dnd->current_prelighted_node 
-	    && dnd->current_prelighted_node != NULL) {
-		nautilus_ctree_node_set_row_style (NAUTILUS_CTREE (tree_view->details->tree), 
-					      dnd->current_prelighted_node,
-					      tree_view->details->dnd->normal_style);
-	}
-	nautilus_ctree_node_set_row_style (NAUTILUS_CTREE (tree_view->details->tree), 
-				      node,
-				      tree_view->details->dnd->highlight_style);
-	dnd->current_prelighted_node = node;
-#endif
+	/* FIXME: We can do better than 10,10 for the hot spot. */
+	eel_drag_set_icon_pixbuf (context, pixbuf, 10, 10);
 }
 
 
 
 /* returns if it was expanded or not */
 static gboolean
-nautilus_tree_view_collapse_node (NautilusCTree *tree, NautilusCTreeNode *node)
+nautilus_tree_view_collapse_node (EelCTree *tree, EelCTreeNode *node)
 {
 	char *node_text;
 	guint8 node_spacing;
-	GdkPixmap *pixmap_closed;
-	GdkBitmap *mask_closed;
-	GdkPixmap *pixmap_opened;
-	GdkBitmap *mask_opened;
+	GdkPixbuf *pixbuf_closed;
+	GdkPixbuf *pixbuf_opened;
 	gboolean is_leaf;
 	gboolean is_expanded;
 
-	nautilus_ctree_get_node_info (NAUTILUS_CTREE(tree), 
+	eel_ctree_get_node_info (EEL_CTREE(tree), 
 				      node, &node_text,
-				      &node_spacing, &pixmap_closed, &mask_closed,
-				      &pixmap_opened, &mask_opened, 
+				      &node_spacing, &pixbuf_closed,
+				      &pixbuf_opened,
 				      &is_leaf, &is_expanded);
 	if (is_expanded) {
 				/* collapse */
-		nautilus_ctree_collapse (NAUTILUS_CTREE(tree),
+		eel_ctree_collapse (EEL_CTREE(tree),
 					 node);
 	}
 
@@ -865,31 +832,29 @@ nautilus_tree_view_collapse_node (NautilusCTree *tree, NautilusCTreeNode *node)
 
 
 static void
-nautilus_tree_view_expand_or_collapse_row (NautilusCTree *tree, int row)
+nautilus_tree_view_expand_or_collapse_row (EelCTree *tree, int row)
 {
-	NautilusCTreeNode *node;
+	EelCTreeNode *node;
 	char *node_text;
 	guint8 node_spacing;
-	GdkPixmap *pixmap_closed;
-	GdkBitmap *mask_closed;
-	GdkPixmap *pixmap_opened;
-	GdkBitmap *mask_opened;
+	GdkPixbuf *pixbuf_closed;
+	GdkPixbuf *pixbuf_opened;
 	gboolean is_leaf;
 	gboolean is_expanded;
 
-	node = nautilus_ctree_node_nth (NAUTILUS_CTREE(tree), row);
-	nautilus_ctree_get_node_info (NAUTILUS_CTREE(tree), 
+	node = eel_ctree_node_nth (EEL_CTREE(tree), row);
+	eel_ctree_get_node_info (EEL_CTREE(tree), 
 				 node, &node_text,
-				 &node_spacing, &pixmap_closed, &mask_closed,
-				 &pixmap_opened, &mask_opened, 
+				 &node_spacing, &pixbuf_closed,
+				 &pixbuf_opened,
 				 &is_leaf, &is_expanded);
 	if (!is_expanded) {
 				/* expand */
-		nautilus_ctree_expand (NAUTILUS_CTREE(tree),
+		eel_ctree_expand (EEL_CTREE(tree),
 				  node);
 	} else {
 				/* collapse */ 
-		nautilus_ctree_collapse (NAUTILUS_CTREE(tree),
+		eel_ctree_collapse (EEL_CTREE(tree),
 				    node);
 	}
 
@@ -907,7 +872,7 @@ nautilus_tree_view_move_copy_files (NautilusTreeView *tree_view,
 	source_uris = NULL;
 	for (p = selection_list; p != NULL; p = p->next) {
 		/* do a shallow copy of all the uri strings of the copied files */
-		source_uris = g_list_prepend (source_uris, ((DragSelectionItem *)p->data)->uri);
+		source_uris = g_list_prepend (source_uris, ((EelDragSelectionItem *)p->data)->uri);
 	}
 	source_uris = g_list_reverse (source_uris);
 	
@@ -930,7 +895,7 @@ nautilus_tree_view_find_drop_target (NautilusTreeView *tree_view,
 {
 	char *target_uri;
 	NautilusFile *file;
-	NautilusCTreeNode *node;
+	EelCTreeNode *node;
 	gboolean is_directory;
 	NautilusTreeNode *current_node;
 	
@@ -958,7 +923,7 @@ nautilus_tree_view_find_drop_target (NautilusTreeView *tree_view,
 
 static gboolean
 nautilus_tree_view_is_tree_node_directory (NautilusTreeView *tree_view,
-					   NautilusCTreeNode *node) 
+					   EelCTreeNode *node) 
 {
 	NautilusTreeNode *model_node;
 	NautilusFile *file;
@@ -975,21 +940,21 @@ nautilus_tree_view_is_tree_node_directory (NautilusTreeView *tree_view,
 
 
 
-static NautilusCTreeNode *
+static EelCTreeNode *
 nautilus_tree_view_tree_node_at (NautilusTreeView *tree_view,
 				 int x, int y) 
 {
 	int row, column, on_row;
-	NautilusCTreeNode *node;
+	EelCTreeNode *node;
 
 
-	on_row = gtk_clist_get_selection_info (GTK_CLIST (tree_view->details->tree), 
-					       x, y, &row, &column);
+	on_row = eel_clist_get_selection_info (EEL_CLIST (tree_view->details->tree), 
+						    x, y, &row, &column);
 
 	node = NULL;
 	if (on_row == 1) {
-		node = nautilus_ctree_node_nth (NAUTILUS_CTREE (tree_view->details->tree),
-					   row);
+		node = eel_ctree_node_nth (EEL_CTREE (tree_view->details->tree),
+						row);
 	}
 
 	return node;
@@ -1009,7 +974,7 @@ static char *
 nautilus_tree_view_item_at (NautilusTreeView *tree_view,
 			    int x, int y)
 {
-	NautilusCTreeNode *node;
+	EelCTreeNode *node;
 
 	node = nautilus_tree_view_tree_node_at (tree_view, x, y);
 	if (node == NULL) {
@@ -1040,14 +1005,14 @@ nautilus_tree_view_ensure_drag_data (NautilusTreeView *tree_view,
 				     GdkDragContext *context,
 				     guint32 time)
 {
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 
 	drag_info = tree_view->details->dnd->drag_info;
 
 	if (!drag_info->got_drop_data_type) {
-		gtk_drag_get_data (GTK_WIDGET (tree_view->details->tree), context,
-				   GPOINTER_TO_INT (context->targets->data),
-				   time);
+		get_data_on_first_target_we_support (GTK_WIDGET (tree_view->details->tree),
+						     context,
+						     time);
 	}
 }
 
@@ -1062,27 +1027,25 @@ nautilus_tree_view_ensure_drag_data (NautilusTreeView *tree_view,
 
 
 static gboolean
-ready_to_start_scrolling (NautilusDragInfo *drag_info,
+ready_to_start_scrolling (EelDragInfo *drag_info,
 			  int y_scroll_delta)
 {
-	return (y_scroll_delta != 0 && drag_info->start_auto_scroll_in < nautilus_get_system_time ()) ||
-		drag_info->start_auto_scroll_in +  AUTOSCROLL_X_ONLY_EXTRA_DELAY < nautilus_get_system_time ();
+	return (y_scroll_delta != 0 && drag_info->start_auto_scroll_in < eel_get_system_time ()) ||
+		drag_info->start_auto_scroll_in +  AUTOSCROLL_X_ONLY_EXTRA_DELAY < eel_get_system_time ();
 }
 
 static int
 auto_scroll_timeout_callback (gpointer data)
 {
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 	NautilusTreeView *tree_view;
-	GtkWidget *widget;
 	float x_scroll_delta, y_scroll_delta;
 	
 	g_assert (NAUTILUS_IS_TREE_VIEW (data));
-	widget = GTK_WIDGET (data);
-	tree_view = NAUTILUS_TREE_VIEW (widget);
+	tree_view = NAUTILUS_TREE_VIEW (data);
 	drag_info = tree_view->details->dnd->drag_info;
 
-	nautilus_drag_autoscroll_calculate_delta (tree_view->details->tree, &x_scroll_delta, &y_scroll_delta);
+	eel_drag_autoscroll_calculate_delta (tree_view->details->tree, &x_scroll_delta, &y_scroll_delta);
 
 	if (drag_info->waiting_to_autoscroll
 	    && !ready_to_start_scrolling (drag_info, y_scroll_delta)) {
@@ -1104,7 +1067,7 @@ nautilus_tree_view_start_auto_scroll (NautilusTreeView *tree_view)
 {
 	g_assert (NAUTILUS_IS_TREE_VIEW (tree_view));
 
-	nautilus_drag_autoscroll_start (tree_view->details->dnd->drag_info,
+	eel_drag_autoscroll_start (tree_view->details->dnd->drag_info,
 					tree_view->details->tree,
 					auto_scroll_timeout_callback,
 					tree_view);
@@ -1116,27 +1079,22 @@ nautilus_tree_view_stop_auto_scroll (NautilusTreeView *tree_view)
 {
 	g_assert (NAUTILUS_IS_TREE_VIEW (tree_view));
 
-        nautilus_drag_autoscroll_stop (tree_view->details->dnd->drag_info);
+        eel_drag_autoscroll_stop (tree_view->details->dnd->drag_info);
 }
 
-
-
 static void
 nautilus_tree_view_real_scroll (NautilusTreeView *tree_view, float delta_x, float delta_y)
 {
 	GtkAdjustment *hadj, *vadj;
 
-	hadj = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (tree_view));
-	vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (tree_view));
-
-	nautilus_gtk_adjustment_set_value (hadj, hadj->value + delta_x);
-	nautilus_gtk_adjustment_set_value (vadj, vadj->value + delta_y);
+	hadj = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (tree_view->details->scrolled_window));
+	vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (tree_view->details->scrolled_window));
 
+	eel_gtk_adjustment_set_value (hadj, hadj->value + delta_x);
+	eel_gtk_adjustment_set_value (vadj, vadj->value + delta_y);
 }
 
 
-
-
 /******************************************
  * Handle the data dropped on the tree view 
  ******************************************/
@@ -1148,7 +1106,7 @@ nautilus_tree_view_get_drop_action (NautilusTreeView *tree_view,
 				    int *default_action,
 				    int *non_default_action)
 {
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 	char *drop_target;
 
 	drag_info = NAUTILUS_TREE_VIEW (tree_view)->details->dnd->drag_info;
@@ -1162,8 +1120,8 @@ nautilus_tree_view_get_drop_action (NautilusTreeView *tree_view,
 
 
 	switch (drag_info->data_type) {
-	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
-	case NAUTILUS_ICON_DND_URI_LIST:
+	case EEL_ICON_DND_GNOME_ICON_LIST:
+	case EEL_ICON_DND_URI_LIST:
 		if (drag_info->selection_list == NULL) {
 			*default_action = 0;
 			*non_default_action = 0;
@@ -1176,14 +1134,14 @@ nautilus_tree_view_get_drop_action (NautilusTreeView *tree_view,
 			*non_default_action = 0;
 			return;
 		}
-		nautilus_drag_default_drop_action_for_icons (context, drop_target, 
+		eel_drag_default_drop_action_for_icons (context, drop_target, 
 							     drag_info->selection_list, 
 							     default_action, 
 							     non_default_action);
 		break;
-	case NAUTILUS_ICON_DND_COLOR:
-	case NAUTILUS_ICON_DND_KEYWORD:	
-	case NAUTILUS_ICON_DND_BGIMAGE:	
+	case EEL_ICON_DND_COLOR:
+	case EEL_ICON_DND_KEYWORD:	
+	case EEL_ICON_DND_BGIMAGE:	
 		/* we handle none of the above */
 		*default_action = context->suggested_action;
 		*non_default_action = context->suggested_action;
@@ -1196,7 +1154,7 @@ nautilus_tree_view_get_drop_action (NautilusTreeView *tree_view,
 
 static void 
 nautilus_tree_view_collapse_all (NautilusTreeView *tree_view,
-				 NautilusCTreeNode *current_node)
+				 EelCTreeNode *current_node)
 {
 	GSList *list, *temp_list;
 
@@ -1204,24 +1162,11 @@ nautilus_tree_view_collapse_all (NautilusTreeView *tree_view,
 	
 
 	for (temp_list = list; temp_list != NULL; temp_list = temp_list->next) {
-		NautilusCTreeNode *expanded_node;
-		expanded_node = (NautilusCTreeNode *) temp_list->data;
-		if (!nautilus_ctree_is_ancestor (NAUTILUS_CTREE (tree_view->details->tree), 
+		EelCTreeNode *expanded_node;
+		expanded_node = (EelCTreeNode *) temp_list->data;
+		if (!eel_ctree_is_ancestor (EEL_CTREE (tree_view->details->tree), 
 						 expanded_node, current_node)) {
-#ifdef MATHIEU_DEBUG
-			{
-				char *expanded_uri, *current_uri;
-				expanded_uri = nautilus_file_get_uri 
-					(nautilus_tree_view_node_to_file (tree_view, expanded_node));
-				current_uri = nautilus_file_get_uri 
-					(nautilus_tree_view_node_to_file (tree_view, current_node));
-
-				g_print ("collapsing %s in %s\n", expanded_uri, current_uri);
-				g_free (expanded_uri);
-				g_free (current_uri);
-			}
-#endif
-			nautilus_tree_view_collapse_node (NAUTILUS_CTREE (tree_view->details->tree), 
+			nautilus_tree_view_collapse_node (EEL_CTREE (tree_view->details->tree), 
 							  expanded_node);
 		}
 	}
@@ -1233,11 +1178,11 @@ nautilus_tree_view_receive_dropped_icons (NautilusTreeView *view,
 					  GdkDragContext *context,
 					  int x, int y)
 {
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 	NautilusTreeView *tree_view;
 	NautilusTreeViewDndDetails *dnd;
 	char *drop_target_uri;
-	NautilusCTreeNode *dropped_node;
+	EelCTreeNode *dropped_node;
 
 	g_assert (NAUTILUS_IS_TREE_VIEW (view));
 
@@ -1253,7 +1198,7 @@ nautilus_tree_view_receive_dropped_icons (NautilusTreeView *view,
 	}
 
 	if (context->action == GDK_ACTION_ASK) {
-		context->action = nautilus_drag_drop_action_ask 
+		context->action = eel_drag_drop_action_ask 
 			(GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
 	}
 
@@ -1261,32 +1206,10 @@ nautilus_tree_view_receive_dropped_icons (NautilusTreeView *view,
 		drop_target_uri = nautilus_tree_view_find_drop_target (tree_view, 
 								       x, y);
 		if (drop_target_uri == NULL) {
-			nautilus_drag_destroy_selection_list (drag_info->selection_list);
+			eel_drag_destroy_selection_list (drag_info->selection_list);
 			return;
 		}
 
-#ifdef MATHIEU_DEBUG
-		{
-			char *action_string;
-			switch (context->action) {
-			case GDK_ACTION_MOVE:
-				action_string = "move";
-				break;
-			case GDK_ACTION_COPY:
-				action_string = "copy";
-				break;
-			case GDK_ACTION_LINK:
-				action_string = "link";
-				break;
-			default:
-				g_assert_not_reached ();
-				action_string = "error";
-				break;
-			}
-			g_print ("%s selection in %s\n", 
-				 action_string, drop_target_uri);
-		}
-#endif
 		nautilus_tree_view_move_copy_files (tree_view, drag_info->selection_list, 
 						    context, drop_target_uri);
 		/* collapse all expanded directories during drag except the one we 
@@ -1304,32 +1227,12 @@ nautilus_tree_view_receive_dropped_icons (NautilusTreeView *view,
 }
 
 
-static void
-nautilus_tree_view_prelight_stop (NautilusTreeView *tree_view)
-{
-#if 0
-	NautilusTreeViewDndDetails *dnd;
-
-	g_assert (NAUTILUS_IS_TREE_VIEW (tree_view));
-
-	dnd = tree_view->details->dnd;
-
-	if (dnd->current_prelighted_node != NULL) {
-		nautilus_ctree_node_set_row_style (NAUTILUS_CTREE (tree_view->details->tree), 
-						   dnd->current_prelighted_node,
-						   tree_view->details->dnd->normal_style);
-	}
-	dnd->current_prelighted_node = NULL;
-#endif
-}
-
-
 
 static void
 nautilus_tree_view_drag_destroy (NautilusTreeView *tree_view)
 {
 	NautilusTreeViewDndDetails *dnd;
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 
 	g_assert (NAUTILUS_IS_TREE_VIEW (tree_view));
 
@@ -1342,9 +1245,8 @@ nautilus_tree_view_drag_destroy (NautilusTreeView *tree_view)
 	nautilus_tree_view_stop_auto_scroll (tree_view);
 
 	/* remove prelighting */
-	nautilus_ctree_set_prelight (NAUTILUS_CTREE (tree_view->details->tree), 
+	eel_ctree_set_prelight (EEL_CTREE (tree_view->details->tree), 
 				     -1);
-	nautilus_tree_view_prelight_stop (tree_view);
 }
 
 
@@ -1352,7 +1254,7 @@ static void
 nautilus_tree_view_drag_destroy_real (NautilusTreeView *tree_view)
 {
 	NautilusTreeViewDndDetails *dnd;
-	NautilusDragInfo *drag_info;
+	EelDragInfo *drag_info;
 
 	g_assert (NAUTILUS_IS_TREE_VIEW (tree_view));
 
@@ -1361,22 +1263,11 @@ nautilus_tree_view_drag_destroy_real (NautilusTreeView *tree_view)
 
 	/* reset booleans used during drag. */
 	drag_info->got_drop_data_type = FALSE;
-	nautilus_drag_destroy_selection_list (drag_info->selection_list);
+	eel_drag_destroy_selection_list (drag_info->selection_list);
 	drag_info->drop_occured = FALSE;
 
-	
-
-	/* misc stuff */
-	/*	
-	if (dnd_info->shadow != NULL) {
-		gtk_object_destroy (GTK_OBJECT (dnd_info->shadow));
-		dnd_info->shadow = NULL;
-	}
-	*/
-
 	if (drag_info->selection_data != NULL) {
-		nautilus_gtk_selection_data_free_deep (drag_info->selection_data);
+		eel_gtk_selection_data_free_deep (drag_info->selection_data);
 		drag_info->selection_data = NULL;
 	}
-
 }
