@@ -2,7 +2,7 @@
 
 /* fm-desktop-icon-view.c - implementation of icon view for managing the desktop.
 
-   Copyright (C) 2000 Eazel, Inc.mou
+   Copyright (C) 2000, 2001 Eazel, Inc.mou
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -21,39 +21,45 @@
 
    Authors: Mike Engber <engber@eazel.com>
    	    Gene Z. Ragan <gzr@eazel.com>
+	    Miguel de Icaza <miguel@ximian.com>
 */
 
 #include <config.h>
 #include "fm-desktop-icon-view.h"
 
+#include <X11/Xatom.h>
+#include <bonobo/bonobo-ui-util.h>
+#include <gtk/gtkmain.h>
 #include <ctype.h>
 #include <dirent.h>
+#include <eel/eel-glib-extensions.h>
+#include <eel/eel-gnome-extensions.h>
+#include <eel/eel-gtk-extensions.h>
+#include <eel/eel-gtk-macros.h>
+#include <eel/eel-stock-dialogs.h>
+#include <eel/eel-string.h>
+#include <eel/eel-vfs-extensions.h>
 #include <fcntl.h>
-#include <bonobo/bonobo-ui-util.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkcheckmenuitem.h>
 #include <libgnome/gnome-dentry.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-mime.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libnautilus-extensions/nautilus-bonobo-extensions.h>
-#include <libnautilus-extensions/nautilus-directory-background.h>
-#include <libnautilus-extensions/nautilus-directory-notify.h>
-#include <libnautilus-extensions/nautilus-file-changes-queue.h>
-#include <libnautilus-extensions/nautilus-file-operations.h>
-#include <libnautilus-extensions/nautilus-file-utilities.h>
-#include <libnautilus-extensions/nautilus-glib-extensions.h>
-#include <libnautilus-extensions/nautilus-global-preferences.h>
-#include <libnautilus-extensions/nautilus-gnome-extensions.h>
-#include <libnautilus-extensions/nautilus-gtk-extensions.h>
-#include <libnautilus-extensions/nautilus-gtk-macros.h>
-#include <libnautilus-extensions/nautilus-link.h>
-#include <libnautilus-extensions/nautilus-metadata.h>
-#include <libnautilus-extensions/nautilus-program-choosing.h>
-#include <libnautilus-extensions/nautilus-stock-dialogs.h>
-#include <libnautilus-extensions/nautilus-string.h>
-#include <libnautilus-extensions/nautilus-trash-monitor.h>
-#include <libnautilus-extensions/nautilus-volume-monitor.h>
+#include <libnautilus-private/nautilus-bonobo-extensions.h>
+#include <libnautilus-private/nautilus-directory-background.h>
+#include <libnautilus-private/nautilus-directory-notify.h>
+#include <libnautilus-private/nautilus-file-changes-queue.h>
+#include <libnautilus-private/nautilus-file-operations.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
+#include <libnautilus-private/nautilus-global-preferences.h>
+#include <libnautilus-private/nautilus-link.h>
+#include <libnautilus-private/nautilus-metadata.h>
+#include <libnautilus-private/nautilus-monitor.h>
+#include <libnautilus-private/nautilus-program-choosing.h>
+#include <libnautilus-private/nautilus-trash-monitor.h>
+#include <libnautilus-private/nautilus-volume-monitor.h>
 #include <limits.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -70,10 +76,22 @@ static const char untranslated_trash_link_name[] = N_("Trash");
 
 #define DESKTOP_BACKGROUND_POPUP_PATH_DISKS	"/popups/background/Before Zoom Items/Volume Items/Disks"
 
+/* Timeout to check the desktop directory for updates */
+#define RESCAN_TIMEOUT 4000
+
+static char *desktop_directory;
+static time_t desktop_dir_modify_time;
+
 struct FMDesktopIconViewDetails
 {
 	BonoboUIComponent *ui;
 	GList *mount_black_list;
+
+	/* For the desktop rescanning
+	 */
+	guint delayed_init_signal;
+	guint reload_desktop_timeout;
+	gboolean pending_rescan;
 };
 
 typedef struct {
@@ -93,8 +111,8 @@ static void     volume_mounted_callback                           (NautilusVolum
 static void     volume_unmounted_callback                         (NautilusVolumeMonitor  *monitor,
 								   NautilusVolume         *volume,
 								   FMDesktopIconView      *icon_view);
-static void     icon_view_create_nautilus_links                   (NautilusIconContainer  *container,
-								   const GList            *item_uris,
+static void     icon_view_handle_uri_list                   	  (NautilusIconContainer  *container,
+								   const char             *item_uris,
 								   int                     x,
 								   int                     y,
 								   FMDirectoryView        *view);
@@ -114,7 +132,7 @@ static void     free_volume_black_list                            (FMDesktopIcon
 static gboolean	volume_link_is_selection 			  (FMDirectoryView 	  *view);
 
 
-NAUTILUS_DEFINE_CLASS_BOILERPLATE (FMDesktopIconView,
+EEL_DEFINE_CLASS_BOILERPLATE (FMDesktopIconView,
 				   fm_desktop_icon_view,
 				   FM_TYPE_ICON_VIEW)
 
@@ -127,6 +145,81 @@ get_icon_container (FMDesktopIconView *icon_view)
 	return NAUTILUS_ICON_CONTAINER (GTK_BIN (icon_view)->child);
 }
 
+static void
+panel_desktop_area_changed (FMDesktopIconView *icon_view)
+{
+	long *borders = NULL;
+	GdkAtom type_returned;
+	int format_returned;
+	unsigned long items_returned;
+	unsigned long bytes_after_return;
+	NautilusIconContainer *icon_container;
+
+	g_return_if_fail (FM_IS_DESKTOP_ICON_VIEW (icon_view));
+
+	icon_container = get_icon_container (icon_view);
+
+	gdk_error_trap_push ();
+	if (XGetWindowProperty (GDK_DISPLAY (),
+				GDK_ROOT_WINDOW (),
+				gdk_atom_intern ("GNOME_PANEL_DESKTOP_AREA",
+						 FALSE),
+				0 /* long_offset */, 
+				4 /* long_length */,
+				False /* delete */,
+				XA_CARDINAL,
+				&type_returned,
+				&format_returned,
+				&items_returned,
+				&bytes_after_return,
+				(unsigned char **)&borders) != Success) {
+		if (borders != NULL)
+			XFree (borders);
+		borders = NULL;
+	}
+			    
+	if (gdk_error_trap_pop ()
+	    || borders == NULL
+	    || type_returned != XA_CARDINAL
+	    || items_returned != 4
+	    || format_returned != 32) {
+		nautilus_icon_container_set_margins (icon_container,
+						     0, 0, 0, 0);
+	} else {
+		nautilus_icon_container_set_margins (icon_container,
+						     borders[0 /* left */],
+						     borders[1 /* right */],
+						     borders[2 /* top */],
+						     borders[3 /* bottom */]);
+	}
+
+	if (borders != NULL)
+		XFree (borders);
+}
+
+static GdkFilterReturn
+desktop_icon_view_property_filter (GdkXEvent *gdk_xevent,
+				   GdkEvent *event,
+				   gpointer data)
+{
+	XEvent *xevent = gdk_xevent;
+	FMDesktopIconView *icon_view;
+
+	icon_view = FM_DESKTOP_ICON_VIEW (data);
+  
+	switch (xevent->type) {
+	case PropertyNotify:
+		if (xevent->xproperty.atom == gdk_atom_intern ("GNOME_PANEL_DESKTOP_AREA", FALSE)) {
+			panel_desktop_area_changed (icon_view);
+		}
+		break;
+	default:
+		break;
+	}
+
+	return GDK_FILTER_CONTINUE;
+}
+
 static void
 fm_desktop_icon_view_destroy (GtkObject *object)
 {
@@ -134,10 +227,20 @@ fm_desktop_icon_view_destroy (GtkObject *object)
 
 	icon_view = FM_DESKTOP_ICON_VIEW (object);
 
+	/* Remove the property filter */
+	gdk_window_remove_filter (GDK_ROOT_PARENT (),
+				  desktop_icon_view_property_filter,
+				  icon_view);
+
+	/* Remove desktop rescan timeout. */
+	if (icon_view->details->reload_desktop_timeout != 0) {
+		gtk_timeout_remove (icon_view->details->reload_desktop_timeout);
+	}
+	
 	/* Delete all of the link files. */
 	delete_all_mount_links ();
 	
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_HOME_URI,
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_HOME_URI,
 					      home_uri_changed,
 					      icon_view);
 
@@ -151,7 +254,7 @@ fm_desktop_icon_view_destroy (GtkObject *object)
 	
 	g_free (icon_view->details);
 
-	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
@@ -216,7 +319,7 @@ fm_desktop_icon_view_handle_middle_click (NautilusIconContainer *icon_container,
 static void
 free_volume_black_list (FMDesktopIconView *icon_view)
 {
-	nautilus_g_list_free_deep (icon_view->details->mount_black_list);
+	eel_g_list_free_deep (icon_view->details->mount_black_list);
 	icon_view->details->mount_black_list = NULL;
 }
 
@@ -239,7 +342,7 @@ volume_in_black_list (FMDesktopIconView *icon_view,
 
 
 static char *
-create_unique_volume_name (const char *desktop_path, const NautilusVolume *volume)
+create_unique_volume_name (const NautilusVolume *volume)
 {
 	GnomeVFSURI *uri;
 	char *uri_path, *new_name;
@@ -254,8 +357,8 @@ create_unique_volume_name (const char *desktop_path, const NautilusVolume *volum
 	index = 1;
 			
 	volume_name = nautilus_volume_monitor_get_volume_name (volume);	
-	
-	uri_path = g_strdup_printf ("%s/%s",desktop_path, volume_name);		
+		
+	uri_path = g_strdup_printf ("%s/%s", desktop_directory, volume_name);		
 	uri = gnome_vfs_uri_new (uri_path);
 	
 	/* Check for existing filename and create a unique name. */
@@ -268,7 +371,7 @@ create_unique_volume_name (const char *desktop_path, const NautilusVolume *volum
 		g_free (new_name);
 		new_name = g_strdup_printf ("%s (%d)", volume_name, index);
 		
-		uri_path = g_strdup_printf ("%s/%s", desktop_path, new_name);
+		uri_path = g_strdup_printf ("%s/%s", desktop_directory, new_name);
 		uri = gnome_vfs_uri_new (uri_path);		
 	}
 	
@@ -294,7 +397,7 @@ static void
 create_mount_link (FMDesktopIconView *icon_view,
 		   const NautilusVolume *volume)
 {
-	char *desktop_path, *target_uri, *volume_name;
+	char *target_uri, *volume_name;
 	const char *icon_name;
 
 	if (volume_in_black_list (icon_view, volume)) {
@@ -305,32 +408,54 @@ create_mount_link (FMDesktopIconView *icon_view,
 	if (!nautilus_volume_monitor_volume_is_removable (volume)) {
 		return;
 	}
-	
+
 	/* Get icon type */
-	switch (volume->type) {
-	case NAUTILUS_VOLUME_CDDA:
-	case NAUTILUS_VOLUME_CDROM:
-		icon_name = "i-cdrom.png";
+	switch (volume->device_type) {
+	case NAUTILUS_DEVICE_AUDIO_CD:
+	case NAUTILUS_DEVICE_CD_ROM_DRIVE:
+		icon_name = "i-cdrom";
 		break;
-		
-	case NAUTILUS_VOLUME_FLOPPY:
-		icon_name = "i-floppy.png";
+
+	case NAUTILUS_DEVICE_CAMERA:
+		icon_name = "i-blockdev";
 		break;
+
 		
+	case NAUTILUS_DEVICE_FLOPPY_DRIVE:
+		icon_name = "i-floppy";
+		break;
+
+	case NAUTILUS_DEVICE_JAZ_DRIVE:
+		icon_name = "i-zipdisk2";
+		break;
+
+	case NAUTILUS_DEVICE_MEMORY_STICK:
+		icon_name = "gnome-ccperiph";
+		break;
+	
+	case NAUTILUS_DEVICE_NFS:
+		icon_name = "i-nfs";
+		break;
+	
+	case NAUTILUS_DEVICE_ZIP_DRIVE:
+		icon_name = "i-zipdisk";
+		break;
+
+	case NAUTILUS_DEVICE_UNKNOWN:
+		icon_name = "i-blockdev";
+		break;	
 	default:
-		icon_name = "i-blockdev.png";
+		icon_name = "i-blockdev";
 		break;	
 	}
 
-	desktop_path = nautilus_get_desktop_directory ();
-	target_uri =  nautilus_volume_monitor_get_target_uri (volume);
+	target_uri = nautilus_volume_monitor_get_target_uri (volume);
 	
-	volume_name = create_unique_volume_name (desktop_path, volume);
+	volume_name = create_unique_volume_name (volume);
 	
 	/* Create link */
-	nautilus_link_local_create (desktop_path, volume_name, icon_name, target_uri, NULL, NAUTILUS_LINK_MOUNT);
+	nautilus_link_local_create (desktop_directory, volume_name, icon_name, target_uri, NULL, NAUTILUS_LINK_MOUNT);
 				    
-	g_free (desktop_path);
 	g_free (target_uri);
 	g_free (volume_name);
 }
@@ -354,6 +479,69 @@ home_uri_changed (gpointer callback_data)
 	update_home_link_and_delete_copies ();
 }
 
+static gboolean
+do_desktop_rescan (gpointer data)
+{
+	FMDesktopIconView *desktop_icon_view;
+	struct stat buf;
+
+	desktop_icon_view = FM_DESKTOP_ICON_VIEW (data);
+	if (desktop_icon_view->details->pending_rescan) {
+		return TRUE;
+	}
+	
+	if (stat (desktop_directory, &buf) == -1) {
+		return TRUE;
+	}
+
+	if (buf.st_ctime == desktop_dir_modify_time) {
+		return TRUE;
+	}
+
+	desktop_icon_view->details->pending_rescan = TRUE;
+
+	nautilus_directory_force_reload (
+		fm_directory_view_get_model (
+			FM_DIRECTORY_VIEW (desktop_icon_view)));
+	return TRUE;
+}
+
+static void
+done_loading (GtkObject *DirectoryView, FMDesktopIconView *desktop_icon_view)
+{
+	struct stat buf;
+
+	desktop_icon_view->details->pending_rescan = FALSE;
+	if (stat (desktop_directory, &buf) == -1) {
+		return;
+	}
+
+	desktop_dir_modify_time = buf.st_ctime;
+}
+
+/* This function is used because the NautilusDirectory model does not
+ * exist always in the desktop_icon_view, so we wait until it has been
+ * instantiated.
+ */
+static void
+delayed_init (FMDesktopIconView *desktop_icon_view)
+{
+	/* Keep track of the load time. */
+	gtk_signal_connect (GTK_OBJECT (fm_directory_view_get_model
+					(FM_DIRECTORY_VIEW (desktop_icon_view))),
+			    "done_loading",
+			    GTK_SIGNAL_FUNC (done_loading), desktop_icon_view);
+
+	/* Monitor desktop directory. */
+	desktop_icon_view->details->reload_desktop_timeout =
+		gtk_timeout_add (RESCAN_TIMEOUT, do_desktop_rescan, desktop_icon_view);
+
+	gtk_signal_disconnect (GTK_OBJECT (desktop_icon_view),
+			       desktop_icon_view->details->delayed_init_signal);
+
+	desktop_icon_view->details->delayed_init_signal = 0;
+}
+
 static void
 fm_desktop_icon_view_initialize (FMDesktopIconView *desktop_icon_view)
 {
@@ -362,11 +550,24 @@ fm_desktop_icon_view_initialize (FMDesktopIconView *desktop_icon_view)
 	GtkAllocation *allocation;
 	GtkAdjustment *hadj, *vadj;
 
+	if (desktop_directory == NULL) {
+		desktop_directory = nautilus_get_desktop_directory ();
+	}
+
 	icon_container = get_icon_container (desktop_icon_view);
 
 	/* Set up details */
 	desktop_icon_view->details = g_new0 (FMDesktopIconViewDetails, 1);	
 
+	/* Do a reload on the desktop if we don't have FAM, a smarter
+	 * way to keep track of the items on the desktop.
+	 */
+	if (!nautilus_monitor_active ()) {
+		desktop_icon_view->details->delayed_init_signal = gtk_signal_connect
+			(GTK_OBJECT (desktop_icon_view), "begin_loading",
+			 GTK_SIGNAL_FUNC (delayed_init), desktop_icon_view);
+	}
+	
 	nautilus_icon_container_set_is_fixed_size (icon_container, TRUE);
 	
 	/* Set up default mount black list */
@@ -383,8 +584,8 @@ fm_desktop_icon_view_initialize (FMDesktopIconView *desktop_icon_view)
 	hadj = GTK_LAYOUT (icon_container)->hadjustment;
 	vadj = GTK_LAYOUT (icon_container)->vadjustment;
 
-	nautilus_gtk_adjustment_set_value (hadj, 0);
-	nautilus_gtk_adjustment_set_value (vadj, 0);
+	eel_gtk_adjustment_set_value (hadj, 0);
+	eel_gtk_adjustment_set_value (vadj, 0);
 
 	fm_directory_view_ignore_hidden_file_preferences
 		(FM_DIRECTORY_VIEW (desktop_icon_view));
@@ -436,19 +637,28 @@ fm_desktop_icon_view_initialize (FMDesktopIconView *desktop_icon_view)
 					GTK_OBJECT (desktop_icon_view));
 	
 	gtk_signal_connect (GTK_OBJECT (icon_container),
-			    "create_nautilus_links",
-			    GTK_SIGNAL_FUNC (icon_view_create_nautilus_links),
+			    "handle_uri_list",
+			    GTK_SIGNAL_FUNC (icon_view_handle_uri_list),
 			    desktop_icon_view);
 
-	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_HOME_URI,
+	eel_preferences_add_callback (NAUTILUS_PREFERENCES_HOME_URI,
 					   home_uri_changed,
 				  	   desktop_icon_view);
+
+	/* Read out the panel desktop area and update the icon container
+	 * accordingly */
+	panel_desktop_area_changed (desktop_icon_view);
+
+	/* Setup the property filter */
+	gdk_window_add_filter (GDK_ROOT_PARENT (),
+			       desktop_icon_view_property_filter,
+			       desktop_icon_view);
 }
 
 static void
 new_terminal_callback (BonoboUIComponent *component, gpointer data, const char *verb)
 {
-	nautilus_gnome_open_terminal (NULL);
+	eel_gnome_open_terminal (NULL);
 }
 
 static void
@@ -457,7 +667,8 @@ change_background_callback (BonoboUIComponent *component,
 			    const char *verb)
 {
 	nautilus_launch_application_from_command 
-		("background-properties-capplet", NULL, FALSE);
+		(_("Background"),
+		 "background-properties-capplet", NULL, FALSE);
 }
 
 static void
@@ -475,7 +686,7 @@ reset_background_callback (BonoboUIComponent *component,
 			   gpointer data, 
 			   const char *verb)
 {
-	nautilus_background_reset 
+	eel_background_reset 
 		(fm_directory_view_get_background (FM_DIRECTORY_VIEW (data)));
 }
 
@@ -527,7 +738,7 @@ trash_link_is_selection (FMDirectoryView *view)
 	
 	selection = fm_directory_view_get_selection (view);
 
-	if (nautilus_g_list_exactly_one_item (selection)
+	if (eel_g_list_exactly_one_item (selection)
 	    && nautilus_file_is_nautilus_link (NAUTILUS_FILE (selection->data))) {
 		uri = nautilus_file_get_uri (NAUTILUS_FILE (selection->data));
 		/* It's probably OK that this only works for local
@@ -557,7 +768,7 @@ volume_link_is_selection (FMDirectoryView *view)
 	
 	selection = fm_directory_view_get_selection (view);
 
-	if (nautilus_g_list_exactly_one_item (selection)
+	if (eel_g_list_exactly_one_item (selection)
 	    && nautilus_file_is_nautilus_link (NAUTILUS_FILE (selection->data))) {
 		uri = nautilus_file_get_uri (NAUTILUS_FILE (selection->data));
 		/* It's probably OK that this only works for local
@@ -581,16 +792,14 @@ fm_desktop_icon_view_trash_state_changed_callback (NautilusTrashMonitor *trash_m
 						   gboolean state,
 						   gpointer callback_data)
 {
-	char *desktop_path, *path;
+	char *path;
 
-	desktop_path = nautilus_get_desktop_directory ();
-	path = nautilus_make_path (desktop_path, TRASH_LINK_NAME);
+	path = nautilus_make_path (desktop_directory, TRASH_LINK_NAME);
 
 	/* Change the XML file to have a new icon. */
-	nautilus_link_local_set_icon (path, state ? "trash-empty.png" : "trash-full.png");
+	nautilus_link_local_set_icon (path, state ? "trash-empty" : "trash-full");
 
 	g_free (path);
-	g_free (desktop_path);
 }
 
 static void
@@ -625,7 +834,7 @@ volume_unmounted_callback (NautilusVolumeMonitor *monitor,
 			   NautilusVolume *volume, 
 			   FMDesktopIconView *icon_view)
 {
-	char *link_path, *desktop_path, *volume_name;
+	char *link_path, *volume_name;
 
 	g_assert (volume != NULL);
 	
@@ -634,43 +843,87 @@ volume_unmounted_callback (NautilusVolumeMonitor *monitor,
 		return;
 	}
 	
-	desktop_path = nautilus_get_desktop_directory ();
-	link_path = nautilus_make_path (desktop_path, volume_name);
+	link_path = nautilus_make_path (desktop_directory, volume_name);
 	unlink_and_notify (link_path);
 
 	g_free (volume_name);
-	g_free (desktop_path);
 	g_free (link_path);
 }
 
 static void
-icon_view_create_nautilus_links (NautilusIconContainer *container, const GList *item_uris,
-			   	 int x, int y, FMDirectoryView *view)
+icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_uris,
+			   int x, int y, FMDirectoryView *view)
 {
-	const GList *element;
-	char *desktop_path;	
+	GList *uri_list, *node;
 	GnomeDesktopEntry *entry;
-	int index;
 	GdkPoint point;
+	char *uri, *local_path;
+	char *stripped_uri;
+	const char *last_slash, *link_name;
 	
 	if (item_uris == NULL) {
 		return;
 	}
 	
-	desktop_path = nautilus_get_desktop_directory ();
 	point.x = x;
 	point.y = y;
-	
-	/* Iterate through all of the URIs in the list */
-	for (element = item_uris, index = 0; element != NULL; element = element->next, index++) {
-		entry = gnome_desktop_entry_load ((char *)element->data);
-		nautilus_link_local_create_from_gnome_entry (entry, desktop_path, &point);
-		gnome_desktop_entry_free (entry);
+		
+	uri_list = gnome_uri_list_extract_uris (item_uris);
+
+	for (node = uri_list; node != NULL; node = node->next) {
+		/* Most of what comes in here is not really URIs, but
+		 * rather paths that have a file: prefix in them.
+		 */
+		uri = eel_make_uri_from_half_baked_uri (node->data);
+
+		/* Make a link using the desktop file contents? */
+		local_path = gnome_vfs_get_local_path_from_uri (uri);
+		if (local_path != NULL) {
+			entry = gnome_desktop_entry_load (local_path);		
+			if (entry != NULL) {
+				/* FIXME: Handle name conflicts? */
+				nautilus_link_local_create_from_gnome_entry (entry, desktop_directory, &point);
+				gnome_desktop_entry_free (entry);
+			}
+			g_free (local_path);
+			if (entry != NULL) {
+				continue;
+			}
+		}
+		
+		/* Make a link from the URI alone. Generate the file
+		 * name by extracting the basename of the URI.
+		 */
+		/* FIXME: This should be using eel_uri_get_basename
+		 * instead of a "roll our own" solution.
+		 */
+		stripped_uri = eel_str_strip_trailing_chr (uri, '/');
+		last_slash = strrchr (stripped_uri, '/');
+		link_name = last_slash == NULL ? NULL : last_slash + 1;
+		
+		if (!eel_str_is_empty (link_name)) {
+			/* FIXME: Handle name conflicts? */
+			nautilus_link_local_create (desktop_directory, link_name,
+						    "gnome-http-url", uri,
+						    &point, NAUTILUS_LINK_GENERIC);
+		}
+		
+		g_free (stripped_uri);
 	}
 	
-	g_free (desktop_path);
+	gnome_uri_list_free_strings (uri_list);
 }
 
+/* update_link_and_delete_copies
+ * 
+ * Look for a particular type of link on the desktop. If the right
+ * link is there, update its target URI. Delete any extra links of
+ * that type.
+ * 
+ * @is_link_function: predicate function to test whether a link is the right type.
+ * @link_name: if non-NULL, only a link with this name is considered a match.
+ * @link_target_uri: new URI to set as link target.
+ */
 static gboolean
 update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 			       const char *link_name,
@@ -678,24 +931,21 @@ update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 {
 	DIR *dir;
 	gboolean found_link;
-	char *desktop_path;
 	struct dirent *dir_entry;
 	char *link_path;
 
-	desktop_path = nautilus_get_desktop_directory ();
-
-	dir = opendir (desktop_path);
+	dir = opendir (desktop_directory);
 	if (dir == NULL) {
-		g_free (desktop_path);
 		return FALSE;
 	}
 
 	found_link = FALSE;
 
 	while ((dir_entry = readdir (dir)) != NULL) {
-		link_path = nautilus_make_path (desktop_path, dir_entry->d_name);
+		link_path = nautilus_make_path (desktop_directory, dir_entry->d_name);
 		if ((* is_link_function) (link_path)) {
-			if (strcmp (dir_entry->d_name, link_name) == 0) {
+			if (!found_link &&
+			     (link_name == NULL || strcmp (dir_entry->d_name, link_name) == 0)) {
 				nautilus_link_local_set_link_uri (link_path, link_target_uri);
 				found_link = TRUE;
 			} else {
@@ -707,8 +957,6 @@ update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 	
 	closedir (dir);
 
-	g_free (desktop_path);
-	
 	return found_link;
 }
 
@@ -720,9 +968,7 @@ update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 static void
 update_home_link_and_delete_copies (void)
 {
-	char *desktop_path, *home_link_name, *home_uri;
-
-	desktop_path = nautilus_get_desktop_directory ();
+	char *home_link_name, *home_uri;
 
 	/* Note to translators: If it's hard to compose a good home
 	 * icon name from the user name, you can use a string without
@@ -734,20 +980,19 @@ update_home_link_and_delete_copies (void)
 	 */
 	home_link_name = g_strdup_printf (_("%s's Home"), g_get_user_name ());
 	
-	home_uri = nautilus_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+	home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
 	
 	if (!update_link_and_delete_copies (nautilus_link_local_is_home_link,
-					    home_link_name,
+					    NULL,
 					    home_uri)) {
-		nautilus_link_local_create (desktop_path,
+		nautilus_link_local_create (desktop_directory,
 					    home_link_name,
-					    "temp-home.png", 
+					    "temp-home", 
 					    home_uri,
 					    NULL,
 					    NAUTILUS_LINK_HOME);
 	}
 	
-	g_free (desktop_path);
 	g_free (home_link_name);
 	g_free (home_uri);
 }
@@ -755,24 +1000,21 @@ update_home_link_and_delete_copies (void)
 static void
 update_trash_link_and_delete_copies (void)
 {
-	char *desktop_path;
 
 	/* Check for trash link */
 	if (update_link_and_delete_copies (nautilus_link_local_is_trash_link,
 					   TRASH_LINK_NAME,
-					   NAUTILUS_TRASH_URI)) {
+					   EEL_TRASH_URI)) {
 		return;
 	}
 
-	desktop_path = nautilus_get_desktop_directory ();
-	nautilus_link_local_create (desktop_path,
+	nautilus_link_local_create (desktop_directory,
 				    TRASH_LINK_NAME,
-				    "trash-empty.png", 
-				    NAUTILUS_TRASH_URI,
+				    "trash-empty", 
+				    EEL_TRASH_URI,
 				    NULL,
-				    NAUTILUS_LINK_TRASH);				    				    
-	g_free (desktop_path);
-	
+				    NAUTILUS_LINK_TRASH);
+
 	/* Make sure link represents current trash state */
 	fm_desktop_icon_view_trash_state_changed_callback (nautilus_trash_monitor_get (),
 						   	   nautilus_trash_monitor_is_empty (),
@@ -857,7 +1099,10 @@ desktop_icons_compare_callback (NautilusIconContainer *container,
 	category_b = get_sort_category (file_b);
 
 	if (category_a == category_b) {
-		return nautilus_file_compare_for_sort (file_a, file_b, NAUTILUS_FILE_SORT_BY_NAME);
+		return nautilus_file_compare_for_sort 
+			(file_a, file_b, NAUTILUS_FILE_SORT_BY_NAME, 
+			 fm_directory_view_should_sort_directories_first (FM_DIRECTORY_VIEW (icon_view)), 
+			 FALSE);
 	}
 
 	/* We know the answer, so prevent the other handlers
@@ -878,7 +1123,7 @@ mount_parameters_new (FMDesktopIconView *view, const char *mount_path)
 	MountParameters *new_parameters;
 
 	g_assert (FM_IS_DESKTOP_ICON_VIEW (view));
-	g_assert (!nautilus_str_is_empty (mount_path)); 
+	g_assert (!eel_str_is_empty (mount_path)); 
 
 	new_parameters = g_new (MountParameters, 1);
 	gtk_object_ref (GTK_OBJECT (view));
@@ -994,7 +1239,7 @@ real_update_menus (FMDirectoryView *view)
 	
 	g_assert (FM_IS_DESKTOP_ICON_VIEW (view));
 
-	NAUTILUS_CALL_PARENT (FM_DIRECTORY_VIEW_CLASS, update_menus, (view));
+	EEL_CALL_PARENT (FM_DIRECTORY_VIEW_CLASS, update_menus, (view));
 
 	desktop_view = FM_DESKTOP_ICON_VIEW (view);
 
@@ -1017,7 +1262,7 @@ real_update_menus (FMDirectoryView *view)
 		 DESKTOP_COMMAND_EMPTY_TRASH_CONDITIONAL,
 		 !include_empty_trash);
 	if (include_empty_trash) {
-		if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_CONFIRM_TRASH)) {
+		if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_CONFIRM_TRASH)) {
 			label = g_strdup (_("Empty Trash..."));
 		} else {
 			label = g_strdup (_("Empty Trash"));
@@ -1067,7 +1312,7 @@ real_merge_menus (FMDirectoryView *view)
 		BONOBO_UI_VERB_END
 	};
 
-	NAUTILUS_CALL_PARENT (FM_DIRECTORY_VIEW_CLASS, merge_menus, (view));
+	EEL_CALL_PARENT (FM_DIRECTORY_VIEW_CLASS, merge_menus, (view));
 
 	desktop_view = FM_DESKTOP_ICON_VIEW (view);
 
