@@ -29,7 +29,7 @@
 #include <glib.h>
 #include <gtk/gtkwindow.h>
 #include <libgnomevfs/gnome-vfs-types.h>
-#include <libnautilus-extensions/nautilus-file.h>
+#include <libnautilus-private/nautilus-file.h>
 
 void fm_report_error_loading_directory	 (NautilusFile   *file,
 					  GnomeVFSResult  error_code,
